/*
Copyright 2004 John Tsiombikas <nuclear@siggraph.org>

This file is part of my common code collection (ccc).

ccc is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

ccc is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with ccc; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "err_msg.h"


static int verbosity = VERB_INFO;
static int log_verbosity = VERB_INFO;

static FILE *log_file;
static char err_buf[512];
static int initialized = 0;

#define MAX_LOG_FNAME	256
static char log_fname[MAX_LOG_FNAME];

static int init() {
	set_log_filename("logfile");
	
	initialized = 1;
	if((log_file = fopen(log_fname, "w"))) {
		setbuf(log_file, 0);
	} else {
		warning("could not init.log for writing");
	}

	return log_file == 0 ? -1 : 0;
}


/** sets the output verbosity to the given value.
 *
 * values:
 * 	- VERB_QUIET = do not output anything
 * 	- VERB_ERR = output only errors
 * 	- VERB_WARN = output errors and warnings
 * 	- VERB_INFO = output errors, warnings and informative messages
 *
 * default setting: VERB_INFO
 */
void set_verbosity(int value) {
	verbosity = value;
}


/** sets the log verbosity to the given value.
 * 
 * values:
 * 	- VERB_QUIET = do not log anything
 * 	- VERB_ERR = log only errors
 * 	- VERB_WARN = log errors and warnings
 * 	- VERB_INFO = log errors, warnings and informative messages
 * 	
 * default setting: VERB_INFO
 */
void set_log_verbosity(int value) {
	log_verbosity = value;
}

/** sets the filename of the log file 
 *
 * default setting: "logfile"
 */
void set_log_filename(const char *fname) {
	strncpy(log_fname, fname, MAX_LOG_FNAME-1);
}

/** printf-like usage */
void error(const char *str, ...) {
	va_list arg_list;

	if(!initialized) init();

	strcpy(err_buf, "E: ");
		
	va_start(arg_list, str);
	vsprintf(err_buf+3, str, arg_list);
	va_end(arg_list);
	
	if(verbosity >= VERB_ERR) {
		fputs(err_buf, stderr);
		fputc('\n', stderr);
	}

	if(log_verbosity >= VERB_ERR && log_file) {
		fputs(err_buf, log_file);
		fputc('\n', log_file);
	}
}

/** printf-like usage */
void warning(const char *str, ...) {
	va_list arg_list;

	if(!initialized) init();

	strcpy(err_buf, "W: ");
	
	va_start(arg_list, str);
	vsprintf(err_buf+3, str, arg_list);
	va_end(arg_list);
	
	if(verbosity >= VERB_WARN) {
		fputs(err_buf, stderr);
		fputc('\n', stderr);
	}

	if(log_verbosity >= VERB_WARN && log_file) {
		fputs(err_buf, log_file);
		fputc('\n', log_file);
	}
}

/** printf-like usage */
void info(const char *str, ...) {
	va_list arg_list;

	if(!initialized) init();

	strcpy(err_buf, "I: ");
	
	va_start(arg_list, str);
	vsprintf(err_buf+3, str, arg_list);
	va_end(arg_list);
	
	if(verbosity >= VERB_INFO) {
		fputs(err_buf, stdout);
		fputc('\n', stdout);
	}

	if(log_verbosity >= VERB_INFO && log_file) {
		fputs(err_buf, log_file);
		fputc('\n', log_file);
	}
}
