/*
Copyright 2004 John Tsiombikas <nuclear@siggraph.org>

This file is part of the MUD project by The Lab Software.

MUD is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

MUD is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with MUD; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <limits.h>
#include "item.h"
#include "err_msg.h"

unsigned int item_max_use_count[ITEM_TYPE_COUNT] = {
	0,			/* unknown */
	UINT_MAX,	/* food */
	UINT_MAX,	/* drink */
	UINT_MAX,	/* gear (this is never checked, instead use the gear_max_use_count table) */
	1,			/* weapon (two weapons style unsupported) */
	1,			/* text */
	1			/* container (one backpack, although they can be nested) */
};

unsigned int gear_max_use_count[GEAR_TYPE_COUNT] = {
	1,			/* head gear */
	2,			/* arm */
	2,			/* hand */
	1,			/* torso */
	1,			/* waist */
	2,			/* leg */
	2,			/* foot */
	1			/* back */
};

/** sets the item name (the string is copied, you don't need to keep the old value) */
void set_item_name(struct item *item, const char *name) {
	char *tmp;
	
	free(item->name);
	if(!(tmp = malloc(strlen(name)+1))) {
		error("could not set the item name: %s\n", strerror(errno));
		return;
	}

	item->name = tmp;
	strcpy(item->name, name);
}

/** sets the item description (the string is copied, you don't need to keep the old value) */
void set_item_desc(struct item *item, const char *desc) {
	char *tmp;
	
	free(item->desc);
	if(!(tmp = malloc(strlen(desc)+1))) {
		error("could not set the item description: %s\n", strerror(errno));
		return;
	}

	item->desc = tmp;
	strcpy(item->desc, desc);
}

/** sets the text contents (the string is copied, you don't need to keep the old value) */
void set_text_contents(struct text *text, const char *contents) {
	char *tmp;
	
	free(text->text);
	if(!(tmp = malloc(strlen(contents)+1))) {
		error("could not set the text: %s\n", strerror(errno));
		return;
	}

	text->text = tmp;
	strcpy(text->text, contents);
}

/**
 * destroy an item.
 * 
 * beware, do not use this on any item that contains additional
 * dynamic allocations except from the standard ones that all the items have
 */
void destroy_item(struct item *item) {
	free(item->name);
	free(item->desc);
	free(item);
}

/** creates and initialized a container item */
struct container *create_container(int size) {
	struct container *new;

	if(!(new = malloc(sizeof(struct container)))) {
		error("could not create container: %s\n", strerror(errno));
		return 0;
	}

	if(!(new->items = malloc(size * sizeof(struct item*)))) {
		error("could not allocate item table in container: %s\n", strerror(errno));
		free(new);
		return 0;
	}

	memset(new->items, 0, size * sizeof(struct item*));
	new->count = 0;
	new->max = size;

	return new;
}

/** destroys a container item */
void destroy_container(struct container *cont) {
	free(cont->name);
	free(cont->desc);
	free(cont->items);
	free(cont);
}
